/* MARS Network Monitoring Engine
   Copyright (C) 1999 Brian H. Trammell
   Copyright (C) 2002 Leapfrog Research & Development, LLC

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program; if not, it is available at 
	http:///www.gnu.org/copyleft/gpl.html, or by writing to the
	Free Software Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA  02111-1307, USA.
*/

package org.altara.mars.engine;

import java.io.*;
import java.net.*;
import java.util.*;
import org.altara.mars.*;

/** SimpleServiceProbe supports send/expect ASCII probing of a service
	on a host. This covers most protocols where the protocol
	headers are plain text, such as HTTP, SMTP, etc.
*/

public class SimpleServiceProbe extends Probe {

	private String send;
	private String expect;

	public SimpleServiceProbe (
			Service service, String send, String expect) {
		super(service);
		this.send = send;
		this.expect = expect;
	}
	
	protected Status doProbe() {
		Socket sock = null;
		StringBuffer buf = null;
		StringReceiver receiver = null;

		try {
			// Create the socket
			try {
				sock = new Socket(service.getHost().getAddress(),
					service.getPort());
			} catch (IOException ex) {
				// no socket means nobody listening. Service is down.
				return new Status(Status.DOWN);
			}

			try {
				// Send the send-string, if necessary
				if (send != null && send.length() > 0) {
					Writer wtr = new OutputStreamWriter(sock.getOutputStream());
					wtr.write(send);
					wtr.flush();
				}	

				// Start the reader thread
				buf = new StringBuffer();
				receiver = new StringReceiver(sock,buf);
				receiver.start();
			} catch (IOException ex) {
				// IO exception during send or receive setup means premature
				// close. Either the network is funky or we're tcpwrapped.
				return new Status(Status.FASTCLOSE);
			}
	
			// Start checking for timeout
			long timeout = service.getTimeout();
			long tostart = System.currentTimeMillis();
			long resptime = 0;
	
			// Keep checking buffer against expected until timeout
			while (tostart + timeout > System.currentTimeMillis()) {
				synchronized (buf) {
					// check for closed receiver
					if (!receiver.isOpen()) {
						break;
					}
					if (buf.toString().indexOf(expect) >= 0) {
						// we're good
						resptime = System.currentTimeMillis() - tostart;
						Status out = new Status(Status.UP, resptime);
						out.setProperty("received",buf.toString());
						return out;
					}
					// wait for more characters in the buffer
					buf.wait();
				}
			}

			// Reached timeout.
			Status out = new Status(Status.TIMEOUT);
			out.setProperty("received",buf.toString());
			return out;

			// the distinction between timeout and unexpected
			// data in the case of a simple service probe
			// is academic. This probe will never return UNEXPECTED.
			/*
			if (buf.length() == 0) {
				return new Status(Status.TIMEOUT);
			} else {
				Status out = new Status(Status.UNEXPECTED);
				out.setProperty("received",buf.toString());
				return out;
			}
			*/
		} catch (InterruptedException ex) {
			// interrupted thread means internal probe failure
			return new Status(Status.PROBEFAIL);
		} finally {
			// make sure the socket gets closed. ignore errors.
			try {
				sock.close();
			} catch (Throwable ignored) {}
		}
	}
	
	// Helper thread to stuff received characters into a string buffer.
	protected class StringReceiver extends Thread {
		private StringBuffer buf;
		private Reader rdr;
		private boolean open;

		protected StringReceiver(Socket sock, StringBuffer buf)
				throws IOException {
			this.rdr = new InputStreamReader(sock.getInputStream());
			this.buf = buf;
			this.open = true;
		}

		public void run() {
			char c;
			try {
				while(true) {
					c = (char)rdr.read();
					synchronized (buf) {
						buf.append(c);
						buf.notify();
					}
				}
			} catch (IOException ex) {
				this.open = false;
				synchronized (buf) {
					buf.notify();
				}
			}
		}

		public boolean isOpen() {
			return this.open;
		}
	}
}
