/* MARS Network Monitor Swing User Interface
   Copyright (C) 1999 Brian H. Trammell
   Copyright (C) 2002 Leapfrog Research & Development, LLC

	This program is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 2
	of the License, or (at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program; if not, it is available at 
	http:///www.gnu.org/copyleft/gpl.html, or by writing to the
	Free Software Foundation, Inc., 59 Temple Place - Suite 330,
	Boston, MA  02111-1307, USA.
*/

package org.altara.mars.swingui;

import org.altara.util.*;
import org.altara.mars.*;
import org.altara.mars.engine.*;
import org.jdom.*;
import java.io.*;
import java.net.*;
import java.util.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.tree.*;
import javax.swing.event.*;
import javax.swing.border.*;

/** MarsView is the root of the MARS Swing user interface.
*/

public class MarsView extends JFrame implements MarsConstants {

	// Toolbar actions
	private Action newAction;
	private Action openAction;
	private Action saveAction;
	private Action startAction;
	private Action stopAction;

	// List and tree models
	private FaultListModel flm;
	private ChangeListModel clm;
	private DefaultTreeModel stm;
	private ServiceTreeChangeAdapter stca;
	private DetailListModel dlm;

	// Context menu support classes
	private ServiceTreeContextMenuSupport stcms;

	// Lists and trees
	private JTree serviceTree;

	// Save/Open files
	private File curDir;
	private File curSaveFile;

	public MarsView () {
		// set the window's title
		super("Mars Network Monitor v."+VERSION);

		// Get the controller (understood to be initialized
		// and to have a bound MarsModel) - we'll use it a lot
		// below
		Controller controller = Controller.getCurrent();
		MarsModel model = controller.getModel();

		// initialize toolbar (required for stca connection)
		JToolBar toolbar = initializeToolbar();

		// set file defaults
		curDir = new File(".");
		curSaveFile = null;

		// create a fault list and bind it to the controller
		flm = new FaultListModel();
		controller.addStatusChangeListener(
			new StatusChangeThreadAdapter(flm));
		JList faultList = new JList(flm);
		JScrollPane faultListSP = new JScrollPane(faultList);
		// set up various list options
		faultListSP.setBorder(new TitledBorder("Current Faults"));
		// set up the renderer for the fault list
		FaultListRenderer flr = new FaultListRenderer();
		faultList.setCellRenderer(flr);

		// create a change log and bind it to the controller
		clm = new ChangeListModel();
		controller.addStatusChangeListener(
			new StatusChangeThreadAdapter(clm));
		JList changeList = new JList(clm);
		JScrollPane changeListSP = new JScrollPane(changeList);
		// set up various list options
		changeListSP.setBorder(new TitledBorder("Recent Changes"));
		// set up the renderer for the change list
		ChangeListRenderer clr = new ChangeListRenderer();
		changeList.setCellRenderer(clr);

		// create a service tree bound to the model
		stm = new DefaultTreeModel(controller.getModel());
		serviceTree = new JTree(stm);
		JScrollPane serviceTreeSP = new JScrollPane(serviceTree);
		// set up various tree options
		serviceTree.setRootVisible(false);
		serviceTreeSP.setBorder(new TitledBorder("Hosts & Services"));
		// add a renderer
		ServiceTreeRenderer str = new ServiceTreeRenderer();
		serviceTree.setCellRenderer(str);
		// use a ServiceTreeChangeAdapter to route change events to the
		// correct places...
		stca = new ServiceTreeChangeAdapter(controller,model,
			serviceTree,saveAction);

		// add context menus to the service tree
		stcms = new ServiceTreeContextMenuSupport(this,serviceTree);

		// create a detail view bound to the controller
		// and the service tree
		dlm = new DetailListModel();
		controller.addProbeListener(new ProbeThreadAdapter(dlm));
		serviceTree.addTreeSelectionListener(dlm);
		JList detailList = new JList(dlm);
		JScrollPane detailListSP = new JScrollPane(detailList);
		// set up various list options
		detailListSP.setBorder(new TitledBorder("Service Detail"));
		// no renderer as of yet - set font to size 10, though.
		detailList.setFont(MarsAbstractRenderer.cellFont);

		// ensure the controller is stopped and set appropriate
		// default enable states
		controller.stop();
		stopAction.setEnabled(false);
		saveAction.setEnabled(false);
		stcms.setEditActionsEnabled(true);

		// stuff it all together into a tab pane
		JSplitPane rightPane = new JSplitPane(
			JSplitPane.VERTICAL_SPLIT, faultListSP, detailListSP);
		JSplitPane frontPane = new JSplitPane(
			JSplitPane.HORIZONTAL_SPLIT, serviceTreeSP, rightPane);
		JTabbedPane tabPane = new JTabbedPane(JTabbedPane.BOTTOM);
		tabPane.addTab("Current",frontPane);
		tabPane.addTab("History",changeListSP);

		// now put it into the window
		getContentPane().setLayout(new BorderLayout());
		getContentPane().add(toolbar,BorderLayout.NORTH);
		getContentPane().add(tabPane,BorderLayout.CENTER);

		// set up the window close listener
		setDefaultCloseOperation(DO_NOTHING_ON_CLOSE);
		addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent we) {
				// for now, do this all in-loop
				// later, we'll want to ask the app master
				// to shut down for us
				if (confirmDiscardChanges()) {
					setVisible(false);
					Controller.getCurrent().stop();
					System.exit(0);
				}
			}
		});

		// rock and roll
		this.setSize(400,320);
		this.show();
		rightPane.setDividerLocation(0.5);
		frontPane.setDividerLocation(0.4);
	}

	private JToolBar initializeToolbar() {
		// create the toolbar
		JToolBar toolbar = new JToolBar();

		newAction = 
			new AbstractAction("New...",	
				IconService.getIcon("mi2_tbnew.gif")) {
			public void actionPerformed(ActionEvent ae) {
				newModel();
			}
		};

		openAction = 
			new AbstractAction("Open...",
				IconService.getIcon("mi2_tbopen.gif")) {
			public void actionPerformed(ActionEvent ae) {
				openModel();
			}
		};

		saveAction =
			new AbstractAction("Save...",
				IconService.getIcon("mi2_tbsave.gif")) {
			public void actionPerformed(ActionEvent ae) {
				saveModel();
			}
		};

		startAction =
			new AbstractAction("Start",
				IconService.getIcon("mi2_tbrun.gif")) {
			public void actionPerformed(ActionEvent ae) {
				startController();
			}
		};

		stopAction = 
			new AbstractAction("Stop",
				IconService.getIcon("mi2_tbstop.gif")) {
			public void actionPerformed(ActionEvent ae) {
				stopController();
			}
		};

		// add actions to the toolbar
		toolbar.add(newAction);
		toolbar.add(openAction);
		toolbar.add(saveAction);
		toolbar.addSeparator();
		toolbar.add(startAction);
		toolbar.add(stopAction);

		return toolbar;
	}

	private void newModel() {
		// don't chuck current model without giving
		// the user a chance to save
		if (!confirmDiscardChanges()) return;

		// disconnect the old model from the UI
		Controller controller = Controller.getCurrent();
		MarsModel oldModel = controller.getModel();
		oldModel.setMarsModelListener(null);

		// create the new model and connect it to the UI
		MarsModel newModel = new MarsModel();
		stca.setModel(newModel);
		stm.setRoot(newModel);
		controller.setModel(newModel);

		// disable the save button
		saveAction.setEnabled(false);
	}

	private void openModel() {
		// don't chuck current model without giving
		// the user a chance to save
		if (!confirmDiscardChanges()) return;

		// get the file to open
		File inFile = selectFileOpen();
		// make sure a file was selected
		if (inFile == null) return;

		try {
			// try to parse it
			MarsModel newModel = MarsModel.fromXMLFile(inFile);
			// disconnect the old model
			MarsModel oldModel = Controller.getCurrent().getModel();
			oldModel.setMarsModelListener(null);
			// connect the new model
			stca.setModel(newModel);
			stm.setRoot(newModel);
			Controller.getCurrent().setModel(newModel);
			// disable the save button
			saveAction.setEnabled(false);
		} catch (JDOMException ex) {
			// report load exception to the user.
			JOptionPane.showMessageDialog(this,
				inFile.getName()+" could not be opened.\n"+
				"The file is not a valid XML file.",
				"File Open Error",JOptionPane.ERROR_MESSAGE);
		} catch (InvalidDocumentException ex) {
			// report load exception to the user.
			JOptionPane.showMessageDialog(this,
				inFile.getName()+" could not be opened.\n"+
				"The file contains the following error: "+ex.getMessage(),
				"File Open Error",JOptionPane.ERROR_MESSAGE);
		} catch (UnknownHostException ex) {
			// report load exception to the user.
			JOptionPane.showMessageDialog(this,
				inFile.getName()+" could not be opened.\n"+
				"The host address "+ex.getMessage()+" could not be resolved.\n"+
				"This may indicate a problem with your domain name servers.",
				"File Open Error",JOptionPane.ERROR_MESSAGE);
		}
	
	}

	private boolean saveModel() {
		// select a file to save to, default to last save
		File outFile = selectFileSave();
		if (outFile == null) return false;
		try {
			// write the output file
			Controller.getCurrent().getModel().toXMLFile(outFile,false);
			// disable the save button
			saveAction.setEnabled(false);
			// signal success
			return true;
		} catch (Exception ex) {
			JOptionPane.showMessageDialog(this,
				outFile.getName()+" could not be saved.\n"+
				"You may not have permission to write the file, or the file\n"+
				"may be open. Try saving to a different file or directory.",
				"File Save Error",JOptionPane.ERROR_MESSAGE);
			return false;
		}
	}

	private void startController() {
		startAction.setEnabled(false);
		stopAction.setEnabled(true);
		stcms.setEditActionsEnabled(false);
		Controller.getCurrent().start();
		// now do necessary UI updates
		serviceTree.repaint();
		flm.updateListCache();
	}

	private void stopController() {
		stopAction.setEnabled(false);
		startAction.setEnabled(true);
		stcms.setEditActionsEnabled(true);
		Controller.getCurrent().stop();
		// now do necessary UI updates
		serviceTree.repaint();
		flm.updateListCache();
	}

	private boolean confirmDiscardChanges() {
		// if the model hasn't changed, assume tacit approval
		if (!saveAction.isEnabled()) return true;

		// otherwise, ask the user
		String changeMsg = "There are unsaved changes to the host and "+
							"service list.\nDo you wish to save them before "+
							"proceeding?";
		int confirmed = JOptionPane.showConfirmDialog(this,changeMsg,
			"Unsaved Changes",JOptionPane.YES_NO_CANCEL_OPTION,
			JOptionPane.WARNING_MESSAGE);
		switch (confirmed) {
			case JOptionPane.YES_OPTION:
				return saveModel();
			case JOptionPane.NO_OPTION:
				return true;
			case JOptionPane.CANCEL_OPTION:
			default:
				return false;
		}
	}

	private File selectFileOpen() {
		// create a file chooser
		JFileChooser chooser = new JFileChooser(curDir);
		// ask for a file to open
		int option = chooser.showOpenDialog(this);
		if (option == JFileChooser.APPROVE_OPTION) {
			// remember selected directory
			curDir = chooser.getCurrentDirectory();
			// and return selected file to open
			return chooser.getSelectedFile();
		} else {
			// no file selected, return null
			return null;
		}
	}

	private File selectFileSave() {
		// create a file chooser
		JFileChooser chooser = new JFileChooser(curDir);
		if (curSaveFile != null) chooser.setSelectedFile(curSaveFile);
		// overwrite check loop
		while (true) {
			int option = chooser.showSaveDialog(this);
			if (option == JFileChooser.APPROVE_OPTION) {
				// check for existing selected file
				if (chooser.getSelectedFile().exists()) {
					int confirm = JOptionPane.showConfirmDialog(this,
						"Overwrite existing "+
						chooser.getSelectedFile().getName()+"?",
						"Confirm File Overwrite",
						JOptionPane.YES_NO_OPTION,
						JOptionPane.WARNING_MESSAGE);
					if (confirm == JOptionPane.YES_OPTION) {
						// confirmed overwrite, so no repeat
						break;
					}
				} else {
					// file doesn't exist, so don't repeat
					break;
				}
			} else {
				// not approved, so return null
				return null;
			}
		}

		// remember selected directory and savefile
		curDir = chooser.getCurrentDirectory();
		curSaveFile = chooser.getSelectedFile();
		// return file
		return curSaveFile;
	}

	public void setDefaultSaveFile(File file) {
		this.curSaveFile = file;
	}

	/** A testing main. Though this main() does initialize the controller
		and hook it up to the user interface properly, it isn't intended
		to be the main application entry point, as this presumes that
		the MarsView UI is the only user-side component running (it doesn't
		configure any user-side plugins).

		Presently, this main() will take in the name of a file containing a
		Mars 2 configuration in XML.
	*/

	public static final void main (String[] args) throws Exception {
		// initialize icon service
		IconService.initialize(MarsView.class,"icons");

		// show the splash screen
		final SplashScreen splashScreen = new SplashScreen();
		// set up a timer to kill it
		javax.swing.Timer splashTimer =
			new javax.swing.Timer(3000,new ActionListener() {
			public void actionPerformed(ActionEvent ae) {
				splashScreen.dispose();
			}
		});
		splashTimer.setCoalesce(false);
		splashTimer.setRepeats(false);
		splashTimer.start();

		// initialize service types
		ProbeFactory.registerFactory(new HttpProbeFactory());
		ProbeFactory.registerFactory(new SmtpProbeFactory());
		ProbeFactory.registerFactory(new ImapProbeFactory());
		ProbeFactory.registerFactory(new Pop3ProbeFactory());
		ProbeFactory.registerFactory(new SshProbeFactory());
		ProbeFactory.registerFactory(new FtpProbeFactory());
	
		// Load the specified configuration file
		MarsModel model = new MarsModel();
		File saveFile = null;
		if (args.length > 0) {
			saveFile = new File(args[0]);
			model = MarsModel.fromXMLFile(saveFile);
		}

		// initialize the controller
		Controller.initialize();
		Controller.getCurrent().setModel(model);

		// initialize the user interface
		MarsView mv = new MarsView();

		// hold on to the save file
		if (saveFile != null) mv.setDefaultSaveFile(saveFile);
	}

	public static class SplashScreen extends JWindow {
		public SplashScreen() {
			super();

			// get the splash graphic
			JLabel splashLabel = new JLabel(IconService.getIcon("splash.gif"));

			// add it to the frame
			getContentPane().add(splashLabel);

			// display it
			this.pack();
			this.centerWindow();
			this.show();
		}
	
		private void centerWindow() {
			Rectangle virtualBounds = new Rectangle();
			GraphicsEnvironment ge =
				GraphicsEnvironment.getLocalGraphicsEnvironment();
			GraphicsDevice[] gs = ge.getScreenDevices();
			for (int j=0;j<gs.length;j++) { 
				GraphicsDevice gd = gs[j];
				GraphicsConfiguration[] gc = gd.getConfigurations();
				for (int i=0; i < gc.length; i++) {
					virtualBounds = virtualBounds.union(gc[i].getBounds());
				}
			}
	
			int cx = (int) virtualBounds.getWidth()/2;
			int cy = (int) virtualBounds.getHeight()/2;
			this.setLocation(cx - this.getWidth()/2,
								cy - this.getHeight()/2);
		}
	}
}
